clc
clear all
close all

%%%%% symbolic quantities
syms theta1 theta2 real
syms omega1 omega2 real
syms alpha1 alpha2 real %angular acceleration
syms m1 I1 m2 I2 g real
syms c1 c2 l real

%%%%% position vectors %%%%
x_G1 = c1*sin(theta1); y_G1 = -c1*cos(theta1);
x_G2 = l*sin(theta1) + c2*sin(theta1+theta2); y_G2 = -l*cos(theta1) -c2*cos(theta1+theta2);

%%%%% velocity vectors %%%%%%
%%% I used jacobian instead of diff.
v_G1_x = jacobian(x_G1,[theta1 theta2])*[omega1 omega2]'; %jacobian will be explained in the next few lectures
v_G1_y = jacobian(y_G1,[theta1 theta2])*[omega1 omega2]'; 
v_G2_x = jacobian(x_G2,[theta1 theta2])*[omega1 omega2]';
v_G2_y = jacobian(y_G2,[theta1 theta2])*[omega1 omega2]';
v_G1 = [v_G1_x; v_G1_y];
v_G2 = [v_G2_x; v_G2_y];

 
%%%% lagrangian %%
%%%% v_G1 = [xdot_G1 ydot_G1]
%%% v_G1' * v_G1 = xdot_G1^2 + ydot_G1^2 %%% (Check this)
T = 0.5*m1*(v_G1'*v_G1) + 0.5*m2*(v_G2'*v_G2) + 0.5*I1*omega1*omega1 + 0.5*I2*(omega1+omega2)*(omega1+omega2);
V = m1*g*y_G1 + m2*g*y_G2;
L = T-V;
disp('copy paste energy in the code');
disp(['KE(i) = ',char(T),';']);
disp(['PE(i) = ',char(V),';']);
disp(['TE(i) = KE(i)+PE(i);']);
disp(' ');

%Derive equations of motion using Euler-Lagrange method
q = [theta1 theta2];
qdot = [omega1 omega2];
qddot = [alpha1 alpha2];

for ii=1:2
    dLdqdot(ii) = diff(L,qdot(ii));
    ddt_dLdqdot(ii) = diff(dLdqdot(ii),q(1))*qdot(1) + diff(dLdqdot(ii),qdot(1))*qddot(1)+...
                     diff(dLdqdot(ii),q(2))*qdot(2) + diff(dLdqdot(ii),qdot(2))*qddot(2);
    dLdq(ii) = diff(L,q(ii));

    EOM(ii) = ddt_dLdqdot(ii) - dLdq(ii);
end

%%%%%%%%% collecting equations as A alpha = b
A = jacobian(EOM,[alpha1 alpha2]);
b(1,1) = -subs(EOM(1),[alpha1 alpha2],[0 0]);
b(2,1) = -subs(EOM(2),[alpha1 alpha2],[0 0]);

disp('copy paste in MATLAB');
disp(' ');

disp(['A11 = ', char(A(1,1)), ';'])
disp(['A12 = ', char(A(1,2)), ';'])
disp(['A21 = ', char(A(2,1)), ';'])
disp(['A22 = ', char(A(2,2)), ';'])

disp(['b1 = ', char(b(1,1)), ';'])
disp(['b2 = ', char(b(2,1)), ';'])

disp('A = [A11 A12; A21 A22];');
disp('b = [b1; b2];');
disp('alpha = A\b;');
